import { useState } from 'react';
import { AxiosRequestConfig } from "axios";
import Request from '@ywfe/request';
// @ts-ignore
import * as querystring from 'querystring';
// @ts-ignore
import { ParsedUrlQueryInput } from 'querystring';
import * as R from 'ramda';
import { LocalStorage } from "node-localstorage";
import {configs} from './config';

const isServer = typeof window === 'undefined';
if(isServer){
  global.localStorage = new LocalStorage('./scratch');
}

<%if (projectId){%>
  const projectId = "<%= projectId%>";
  
  const env = localStorage.getItem('env') || (process.env.REACT_APP_ENV === 'production' ? 'prod': process.env.REACT_APP_ENV) || 'debug';
  const basePath = configs[projectId][env].API_BASE_URL + `${env === 'mock' ? '/' + <%= project_id %> : ''}`;
  <% } else { %>
    const basePath = '';
  <% } %>

// @ts-ignore
const parseParams = <T>(params: T) => {
  return R.values<any, any>(params).join('/');
}

export interface <%= name%>Input {
  <% for(let i = 0; i < inputs.length; i++) {%>
    <%= inputs[i]%>
  <% } %>
}

export interface <%= name%>Data {
  <% if (responses && responses.length) {%>
    <% for(let i = 0; i < responses.length; i++) {%>
      <%= responses[i]%>
    <% } %>
  <% } %>
}

// <%= id%> <%= title%>
export const use<%= classify(dasherize(method))%><%= name%> = () => {
  const [is<%= name%>Loading, setIs<%= name%>Loading] = useState(false);
  const [data<%= name%>, setData<%= name%>] = useState<<%= name%>Data>({} as <%= name%>Data);
  const <%= dasherize(method)%><%= name%>Data = async(<%if (inputs && inputs.length) {%>params: <%= name%>Input,<% } %> config?: AxiosRequestConfig): Promise<<%= name%>Data> => {
    setIs<%= name%>Loading(true);
    try{
      <% if ( isRestful ) { %>
        const res: any = await Request.<%= dasherize(method)%><<%= name%>Data>(`${basePath}<%= path%>/<% if (inputs && inputs.length) { %>${parseParams<<%= name%>Input>(params)}<% }%>`, config);
        <% } else { %>
        <% if (method === 'GET') { %>
        const res: any = await Request.get<<%= name%>Data>(`${basePath}<%= path%><% if (inputs && inputs.length) { %>?${querystring.stringify(params as unknown as ParsedUrlQueryInput)}<% }%>`, config);
        <% } %>
        <% if (method === 'POST') { %>
        const res: any = await Request.post<<%= name%>Data>(`${basePath}<%= path%>`, <% if (inputs && inputs.length) { %>params<% } else {%>{}<% }%>, config);
        <% } %>
        <% } %>
        
        setData<%= name%>(res || {});
        setIs<%= name%>Loading(false);
        return res;
    } catch(error){
      console.log(error);
      setIs<%= name%>Loading(false);
      return false as <%= name%>Data;
    }
    
  }

  return {
    is<%= name%>Loading,
    data<%= name%>,
    <%= dasherize(method)%><%= name%>Data,
  }
}